@@ -12,8 +12,7 @@
 //                                                            ///
 /////////////////////////////////////////////////////////////////
 
-
-//die('Due to a security issue, this demo has been disabled. It can be enabled by removing line '.__LINE__.' in demos/'.basename(__FILE__));
+die('Due to a security issue, this demo has been disabled. It can be enabled by removing line '.__LINE__.' in demos/'.basename(__FILE__));
 
 
 /////////////////////////////////////////////////////////////////
@@ -51,11 +50,12 @@ $getID3checkColor_UnknownDark    = 'BBBBDD';
 ///////////////////////////////////////////////////////////////////////////////
 
 
-header('Content-Type: text/html; charset=UTF-8');
+header('Content-Type: text/html; charset='.$PageEncoding);
 echo '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
 echo '<html><head>';
 echo '<title>getID3() - /demo/demo.browse.php (sample script)</title>';
 echo '<link rel="stylesheet" href="getid3.css" type="text/css">';
+echo '<meta http-equiv="Content-Type" content="text/html;charset='.$PageEncoding.'" />';
 echo '</head><body>';
 
 if (isset($_REQUEST['deletefile'])) {
@@ -112,6 +112,7 @@ if (isset($_REQUEST['filename'])) {
 		echo 'Browse: <a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory), ENT_QUOTES).'">'.getid3_lib::iconv_fallback('ISO-8859-1', 'UTF-8', $listdirectory).'</a><br>';
 	}
 
+	getid3_lib::ksort_recursive($ThisFileInfo);
 	echo table_var_dump($ThisFileInfo, false, $PageEncoding);
 	$endtime = microtime(true);
 	echo 'File parsed in '.number_format($endtime - $starttime, 3).' seconds.<br>';
@@ -286,11 +287,11 @@ if (isset($_REQUEST['filename'])) {
 					echo '<td align="left">&nbsp;';
 					if (!empty($fileinfo['warning'])) {
 						$FilesWithWarnings++;
-						echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['warning']), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['warning']), ENT_QUOTES).'">warning</a><br>';
+						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['warning']))), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['warning']), ENT_QUOTES).'">warning</a><br>';
 					}
 					if (!empty($fileinfo['error'])) {
 						$FilesWithErrors++;
-						echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['error']), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['error']), ENT_QUOTES).'">error</a><br>';
+						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['error']))),   ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['error']),   ENT_QUOTES).'">error</a><br>';
 					}
 					echo '</td>';
 
@@ -441,7 +442,6 @@ function string_var_dump($variable) {
 	return $dumpedvariable;
 }
 
-
 function table_var_dump($variable, $wrap_in_td=false, $encoding='ISO-8859-1') {
 	$returnstring = '';
 	switch (gettype($variable)) {
@@ -449,7 +449,7 @@ function table_var_dump($variable, $wrap_in_td=false, $encoding='ISO-8859-1') {
 			$returnstring .= ($wrap_in_td ? '<td>' : '');
 			$returnstring .= '<table class="dump" cellspacing="0" cellpadding="2">';
 			foreach ($variable as $key => $value) {
-				$returnstring .= '<tr><td valign="top"><b>'.str_replace("\x00", ' ', $key).'</b></td>';
+				$returnstring .= '<tr><td valign="top"><b>'.str_replace("\x00", ' ', $key).'</b></td>'."\n";
 				$returnstring .= '<td valign="top">'.gettype($value);
 				if (is_array($value)) {
 					$returnstring .= '&nbsp;('.count($value).')';
@@ -460,40 +460,41 @@ function table_var_dump($variable, $wrap_in_td=false, $encoding='ISO-8859-1') {
 				if (($key == 'data') && isset($variable['image_mime'])) {
 					$imageinfo = array();
 					$imagechunkcheck = getid3_lib::GetDataImageSize($value, $imageinfo);
-					$returnstring .= '</td><td><img src="data:image/gif;base64,'.base64_encode($value).'" width="'.$imagechunkcheck[0].'" height="'.$imagechunkcheck[1].'"></td></tr>';
+					$returnstring .= '</td>'."\n".'<td><img src="data:'.$variable['image_mime'].';base64,'.base64_encode($value).'" width="'.$imagechunkcheck[0].'" height="'.$imagechunkcheck[1].'"></td></tr>'."\n";
 				} else {
-					$returnstring .= '</td>'.table_var_dump($value, true, $encoding).'</tr>';
+					$returnstring .= '</td>'."\n".table_var_dump($value, true, $encoding).'</tr>'."\n";
 				}
 			}
-			$returnstring .= '</table>';
-			$returnstring .= ($wrap_in_td ? '</td>' : '');
+			$returnstring .= '</table>'."\n";
+			$returnstring .= ($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		case 'boolean':
-			$returnstring .= ($wrap_in_td ? '<td class="dump_boolean">' : '').($variable ? 'TRUE' : 'FALSE').($wrap_in_td ? '</td>' : '');
+			$returnstring .= ($wrap_in_td ? '<td class="dump_boolean">' : '').($variable ? 'TRUE' : 'FALSE').($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		case 'integer':
-			$returnstring .= ($wrap_in_td ? '<td class="dump_integer">' : '').$variable.($wrap_in_td ? '</td>' : '');
+			$returnstring .= ($wrap_in_td ? '<td class="dump_integer">' : '').$variable.($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		case 'double':
 		case 'float':
-			$returnstring .= ($wrap_in_td ? '<td class="dump_double">' : '').$variable.($wrap_in_td ? '</td>' : '');
+			$returnstring .= ($wrap_in_td ? '<td class="dump_double">' : '').$variable.($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		case 'object':
 		case 'null':
-			$returnstring .= ($wrap_in_td ? '<td>' : '').string_var_dump($variable).($wrap_in_td ? '</td>' : '');
+			$returnstring .= ($wrap_in_td ? '<td>' : '').string_var_dump($variable).($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		case 'string':
-			$variable = str_replace("\x00", ' ', $variable);
-			$varlen = strlen($variable);
-			for ($i = 0; $i < $varlen; $i++) {
-				$returnstring .= htmlentities($variable{$i}, ENT_QUOTES, $encoding);
-			}
-			$returnstring = ($wrap_in_td ? '<td class="dump_string">' : '').nl2br($returnstring).($wrap_in_td ? '</td>' : '');
+			//$variable = str_replace("\x00", ' ', $variable);
+			//$varlen = strlen($variable);
+			//for ($i = 0; $i < $varlen; $i++) {
+			//	$returnstring .= htmlentities($variable{$i}, ENT_QUOTES, $encoding);
+			//}
+			$returnstring = htmlentities($variable, ENT_QUOTES, $encoding);
+			$returnstring = ($wrap_in_td ? '<td class="dump_string">' : '').nl2br($returnstring).($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
 		default:
@@ -502,13 +503,13 @@ function table_var_dump($variable, $wrap_in_td=false, $encoding='ISO-8859-1') {
 			if (($imagechunkcheck[2] >= 1) && ($imagechunkcheck[2] <= 3)) {
 				$returnstring .= ($wrap_in_td ? '<td>' : '');
 				$returnstring .= '<table class="dump" cellspacing="0" cellpadding="2">';
-				$returnstring .= '<tr><td><b>type</b></td><td>'.getid3_lib::ImageTypesLookup($imagechunkcheck[2]).'</td></tr>';
-				$returnstring .= '<tr><td><b>width</b></td><td>'.number_format($imagechunkcheck[0]).' px</td></tr>';
-				$returnstring .= '<tr><td><b>height</b></td><td>'.number_format($imagechunkcheck[1]).' px</td></tr>';
-				$returnstring .= '<tr><td><b>size</b></td><td>'.number_format(strlen($variable)).' bytes</td></tr></table>';
-				$returnstring .= ($wrap_in_td ? '</td>' : '');
+				$returnstring .= '<tr><td><b>type</b></td><td>'.getid3_lib::ImageTypesLookup($imagechunkcheck[2]).'</td></tr>'."\n";
+				$returnstring .= '<tr><td><b>width</b></td><td>'.number_format($imagechunkcheck[0]).' px</td></tr>'."\n";
+				$returnstring .= '<tr><td><b>height</b></td><td>'.number_format($imagechunkcheck[1]).' px</td></tr>'."\n";
+				$returnstring .= '<tr><td><b>size</b></td><td>'.number_format(strlen($variable)).' bytes</td></tr></table>'."\n";
+				$returnstring .= ($wrap_in_td ? '</td>'."\n" : '');
 			} else {
-				$returnstring .= ($wrap_in_td ? '<td>' : '').nl2br(htmlspecialchars(str_replace("\x00", ' ', $variable))).($wrap_in_td ? '</td>' : '');
+				$returnstring .= ($wrap_in_td ? '<td>' : '').nl2br(htmlspecialchars(str_replace("\x00", ' ', $variable))).($wrap_in_td ? '</td>'."\n" : '');
 			}
 			break;
 	}
@@ -586,10 +587,11 @@ function MoreNaturalSort($ar1, $ar2) {
 }
 
 function PoweredBygetID3($string='') {
+	global $getID3;
 	if (!$string) {
 		$string = '<div style="border: 1px #CCCCCC solid; padding: 5px; margin: 5px 0px; float: left; background-color: #EEEEEE; font-size: 8pt; font-face: sans-serif;">Powered by <a href="http://getid3.sourceforge.net"><b>getID3() v<!--GETID3VER--></b><br>http://getid3.sourceforge.net</a><br>Running on PHP v'.phpversion().' ('.(ceil(log(PHP_INT_MAX, 2)) + 1).'-bit)</div>';
 	}
-	return str_replace('<!--GETID3VER-->', GETID3_VERSION, $string);
+	return str_replace('<!--GETID3VER-->', $getID3->version(), $string);
 }
 
 ?>
\ No newline at end of file
