@@ -14,16 +14,19 @@
 /////////////////////////////////////////////////////////////////
 
 
-class getid3_bmp
+class getid3_bmp extends getid3_handler
 {
+	var $ExtractPalette = false;
+	var $ExtractData    = false;
 
-	function getid3_bmp(&$fd, &$ThisFileInfo, $ExtractPalette=false, $ExtractData=false) {
+	function Analyze() {
+		$info = &$this->getid3->info;
 
-	    // shortcuts
-	    $ThisFileInfo['bmp']['header']['raw'] = array();
-	    $thisfile_bmp                         = &$ThisFileInfo['bmp'];
-	    $thisfile_bmp_header                  = &$thisfile_bmp['header'];
-	    $thisfile_bmp_header_raw              = &$thisfile_bmp_header['raw'];
+		// shortcuts
+		$info['bmp']['header']['raw'] = array();
+		$thisfile_bmp                         = &$info['bmp'];
+		$thisfile_bmp_header                  = &$thisfile_bmp['header'];
+		$thisfile_bmp_header_raw              = &$thisfile_bmp_header['raw'];
 
 		// BITMAPFILEHEADER [14 bytes] - http://msdn.microsoft.com/library/en-us/gdi/bitmaps_62uq.asp
 		// all versions
@@ -33,17 +36,18 @@ class getid3_bmp
 		// WORD    bfReserved2;
 		// DWORD   bfOffBits;
 
-		fseek($fd, $ThisFileInfo['avdataoffset'], SEEK_SET);
+		fseek($this->getid3->fp, $info['avdataoffset'], SEEK_SET);
 		$offset = 0;
-		$BMPheader = fread($fd, 14 + 40);
+		$BMPheader = fread($this->getid3->fp, 14 + 40);
 
 		$thisfile_bmp_header_raw['identifier']  = substr($BMPheader, $offset, 2);
 		$offset += 2;
 
-		if ($thisfile_bmp_header_raw['identifier'] != 'BM') {
-			$ThisFileInfo['error'][] = 'Expecting "BM" at offset '.$ThisFileInfo['avdataoffset'].', found "'.$thisfile_bmp_header_raw['identifier'].'"';
-			unset($ThisFileInfo['fileformat']);
-			unset($ThisFileInfo['bmp']);
+		$magic = 'BM';
+		if ($thisfile_bmp_header_raw['identifier'] != $magic) {
+			$info['error'][] = 'Expecting "'.getid3_lib::PrintHexBytes($magic).'" at offset '.$info['avdataoffset'].', found "'.getid3_lib::PrintHexBytes($thisfile_bmp_header_raw['identifier']).'"';
+			unset($info['fileformat']);
+			unset($info['bmp']);
 			return false;
 		}
 
@@ -81,16 +85,16 @@ class getid3_bmp
 			$thisfile_bmp['type_os']      = 'Windows';
 			$thisfile_bmp['type_version'] = 5;
 		} else {
-			$ThisFileInfo['error'][] = 'Unknown BMP subtype (or not a BMP file)';
-			unset($ThisFileInfo['fileformat']);
-			unset($ThisFileInfo['bmp']);
+			$info['error'][] = 'Unknown BMP subtype (or not a BMP file)';
+			unset($info['fileformat']);
+			unset($info['bmp']);
 			return false;
 		}
 
-		$ThisFileInfo['fileformat']                  = 'bmp';
-		$ThisFileInfo['video']['dataformat']         = 'bmp';
-		$ThisFileInfo['video']['lossless']           = true;
-		$ThisFileInfo['video']['pixel_aspect_ratio'] = (float) 1;
+		$info['fileformat']                  = 'bmp';
+		$info['video']['dataformat']         = 'bmp';
+		$info['video']['lossless']           = true;
+		$info['video']['pixel_aspect_ratio'] = (float) 1;
 
 		if ($thisfile_bmp['type_os'] == 'OS/2') {
 
@@ -112,10 +116,10 @@ class getid3_bmp
 			$thisfile_bmp_header_raw['bits_per_pixel'] = getid3_lib::LittleEndian2Int(substr($BMPheader, $offset, 2));
 			$offset += 2;
 
-			$ThisFileInfo['video']['resolution_x']    = $thisfile_bmp_header_raw['width'];
-			$ThisFileInfo['video']['resolution_y']    = $thisfile_bmp_header_raw['height'];
-			$ThisFileInfo['video']['codec']           = 'BI_RGB '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
-			$ThisFileInfo['video']['bits_per_sample'] = $thisfile_bmp_header_raw['bits_per_pixel'];
+			$info['video']['resolution_x']    = $thisfile_bmp_header_raw['width'];
+			$info['video']['resolution_y']    = $thisfile_bmp_header_raw['height'];
+			$info['video']['codec']           = 'BI_RGB '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
+			$info['video']['bits_per_sample'] = $thisfile_bmp_header_raw['bits_per_pixel'];
 
 			if ($thisfile_bmp['type_version'] >= 2) {
 				// DWORD  Compression;      /* Bitmap compression scheme */
@@ -162,9 +166,9 @@ class getid3_bmp
 				$thisfile_bmp_header_raw['identifier']       = getid3_lib::LittleEndian2Int(substr($BMPheader, $offset, 4));
 				$offset += 4;
 
-				$thisfile_bmp_header['compression']             = $this->BMPcompressionOS2Lookup($thisfile_bmp_header_raw['compression']);
+				$thisfile_bmp_header['compression'] = $this->BMPcompressionOS2Lookup($thisfile_bmp_header_raw['compression']);
 
-				$ThisFileInfo['video']['codec'] = $thisfile_bmp_header['compression'].' '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
+				$info['video']['codec'] = $thisfile_bmp_header['compression'].' '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
 			}
 
 		} elseif ($thisfile_bmp['type_os'] == 'Windows') {
@@ -185,6 +189,8 @@ class getid3_bmp
 			// DWORD  biClrUsed;
 			// DWORD  biClrImportant;
 
+			// possibly integrate this section and module.audio-video.riff.php::ParseBITMAPINFOHEADER() ?
+
 			$thisfile_bmp_header_raw['width']            = getid3_lib::LittleEndian2Int(substr($BMPheader, $offset, 4), true);
 			$offset += 4;
 			$thisfile_bmp_header_raw['height']           = getid3_lib::LittleEndian2Int(substr($BMPheader, $offset, 4), true);
@@ -207,14 +213,14 @@ class getid3_bmp
 			$offset += 4;
 
 			$thisfile_bmp_header['compression']  = $this->BMPcompressionWindowsLookup($thisfile_bmp_header_raw['compression']);
-			$ThisFileInfo['video']['resolution_x']    = $thisfile_bmp_header_raw['width'];
-			$ThisFileInfo['video']['resolution_y']    = $thisfile_bmp_header_raw['height'];
-			$ThisFileInfo['video']['codec']           = $thisfile_bmp_header['compression'].' '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
-			$ThisFileInfo['video']['bits_per_sample'] = $thisfile_bmp_header_raw['bits_per_pixel'];
+			$info['video']['resolution_x']    = $thisfile_bmp_header_raw['width'];
+			$info['video']['resolution_y']    = $thisfile_bmp_header_raw['height'];
+			$info['video']['codec']           = $thisfile_bmp_header['compression'].' '.$thisfile_bmp_header_raw['bits_per_pixel'].'-bit';
+			$info['video']['bits_per_sample'] = $thisfile_bmp_header_raw['bits_per_pixel'];
 
 			if (($thisfile_bmp['type_version'] >= 4) || ($thisfile_bmp_header_raw['compression'] == 3)) {
 				// should only be v4+, but BMPs with type_version==1 and BI_BITFIELDS compression have been seen
-				$BMPheader .= fread($fd, 44);
+				$BMPheader .= fread($this->getid3->fp, 44);
 
 				// BITMAPV4HEADER - [44 bytes] - http://msdn.microsoft.com/library/en-us/gdi/bitmaps_2k1e.asp
 				// Win95+, WinNT4.0+
@@ -256,7 +262,7 @@ class getid3_bmp
 			}
 
 			if ($thisfile_bmp['type_version'] >= 5) {
-				$BMPheader .= fread($fd, 16);
+				$BMPheader .= fread($this->getid3->fp, 16);
 
 				// BITMAPV5HEADER - [16 bytes] - http://msdn.microsoft.com/library/en-us/gdi/bitmaps_7c36.asp
 				// Win98+, Win2000+
@@ -276,13 +282,13 @@ class getid3_bmp
 
 		} else {
 
-			$ThisFileInfo['error'][] = 'Unknown BMP format in header.';
+			$info['error'][] = 'Unknown BMP format in header.';
 			return false;
 
 		}
 
 
-		if ($ExtractPalette || $ExtractData) {
+		if ($this->ExtractPalette || $this->ExtractData) {
 			$PaletteEntries = 0;
 			if ($thisfile_bmp_header_raw['bits_per_pixel'] < 16) {
 				$PaletteEntries = pow(2, $thisfile_bmp_header_raw['bits_per_pixel']);
@@ -290,7 +296,7 @@ class getid3_bmp
 				$PaletteEntries = $thisfile_bmp_header_raw['colors_used'];
 			}
 			if ($PaletteEntries > 0) {
-				$BMPpalette = fread($fd, 4 * $PaletteEntries);
+				$BMPpalette = fread($this->getid3->fp, 4 * $PaletteEntries);
 				$paletteoffset = 0;
 				for ($i = 0; $i < $PaletteEntries; $i++) {
 					// RGBQUAD          - http://msdn.microsoft.com/library/en-us/gdi/bitmaps_5f8y.asp
@@ -311,12 +317,13 @@ class getid3_bmp
 			}
 		}
 
-		if ($ExtractData) {
-			fseek($fd, $thisfile_bmp_header_raw['data_offset'], SEEK_SET);
+		if ($this->ExtractData) {
+			fseek($this->getid3->fp, $thisfile_bmp_header_raw['data_offset'], SEEK_SET);
 			$RowByteLength = ceil(($thisfile_bmp_header_raw['width'] * ($thisfile_bmp_header_raw['bits_per_pixel'] / 8)) / 4) * 4; // round up to nearest DWORD boundry
-			$BMPpixelData = fread($fd, $thisfile_bmp_header_raw['height'] * $RowByteLength);
+			$BMPpixelData = fread($this->getid3->fp, $thisfile_bmp_header_raw['height'] * $RowByteLength);
 			$pixeldataoffset = 0;
-			switch (@$thisfile_bmp_header_raw['compression']) {
+			$thisfile_bmp_header_raw['compression'] = (isset($thisfile_bmp_header_raw['compression']) ? $thisfile_bmp_header_raw['compression'] : '');
+			switch ($thisfile_bmp_header_raw['compression']) {
 
 				case 0: // BI_RGB
 					switch ($thisfile_bmp_header_raw['bits_per_pixel']) {
@@ -398,7 +405,7 @@ class getid3_bmp
 							break;
 
 						default:
-							$ThisFileInfo['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
+							$info['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
 							break;
 					}
 					break;
@@ -473,7 +480,7 @@ class getid3_bmp
 							break;
 
 						default:
-							$ThisFileInfo['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
+							$info['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
 							break;
 					}
 					break;
@@ -562,7 +569,7 @@ class getid3_bmp
 							break;
 
 						default:
-							$ThisFileInfo['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
+							$info['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
 							break;
 					}
 					break;
@@ -602,14 +609,14 @@ class getid3_bmp
 							break;
 
 						default:
-							$ThisFileInfo['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
+							$info['error'][] = 'Unknown bits-per-pixel value ('.$thisfile_bmp_header_raw['bits_per_pixel'].') - cannot read pixel data';
 							break;
 					}
 					break;
 
 
 				default: // unhandled compression type
-					$ThisFileInfo['error'][] = 'Unknown/unhandled compression type value ('.$thisfile_bmp_header_raw['compression'].') - cannot decompress pixel data';
+					$info['error'][] = 'Unknown/unhandled compression type value ('.$thisfile_bmp_header_raw['compression'].') - cannot decompress pixel data';
 					break;
 			}
 		}
